package com.seeyon.apps.ocip.exchange.collaboration.aspect;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.seeyon.ctp.common.trace.vo.TraceWebVO;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.Strings;
import org.aspectj.lang.ProceedingJoinPoint;

/**
 * TraceDao 的aop
 *
 * @author renx 2019-6-5
 */
public class OcipTraceDaoAspect {

    /**
     * OCIP-1816 单组织A 撤销回退记录中 按照撤销/回退人查询时，默认查询到的非全部数据
     * com.seeyon.ctp.common.trace.dao.TraceDao#findDataByParam(com.seeyon.ctp.util.FlipInfo, java.lang.String, java.util.Map)
     *
     * @param pjd
     * @return
     * @throws Throwable
     */
    public Object findDataByParam(ProceedingJoinPoint pjd) throws Throwable {
        Object[] args = pjd.getArgs();
        if (args.length >= 3 && args[2] instanceof Map) {
            Map queryCondition = (Map) args[2];
            boolean isSenderName = queryCondition.containsKey("senderName");
            String senderName = (String) queryCondition.get("senderName");
            boolean isOperationName = queryCondition.containsKey("operationName");
            String operationName = (String) queryCondition.get("operationName");
            if (isSenderName || isOperationName) {
                // 改变查询条件
                queryCondition.remove("senderName");
                queryCondition.remove("operationName");
                // 调用被拦截的方法
                FlipInfo result = (FlipInfo) pjd.proceed();
                List<TraceWebVO> modelList = result.getData();
                Iterator<TraceWebVO> iterator = modelList.iterator();
                while (iterator.hasNext()) {
                    TraceWebVO vo = iterator.next();
                    // 根据查询条件进行过滤
                    if (Strings.isNotBlank(senderName)) {
                        String sName = vo.getSenderName();
                        if (Strings.isBlank(sName) || !sName.toLowerCase().contains(senderName.toLowerCase())) {
                            iterator.remove();
                            continue;
                        }
                    }
                    if (Strings.isNotBlank(operationName)) {
                        String opName = vo.getOperationName();
                        if (Strings.isBlank(opName) || !opName.toLowerCase().contains(operationName.toLowerCase())) {
                            iterator.remove();
                            continue;
                        }
                    }
                }
                return result;
            }
        }
        return pjd.proceed();
    }
}
