package com.seeyon.apps.ocip.exchange.collaboration.handler;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.UUID;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.collaboration.util.ColUtil;
import com.seeyon.apps.ocip.exchange.collaboration.manager.CollaborationExchangeManager;
import com.seeyon.apps.ocip.util.CommonUtil;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.usermessage.MessageReceiver;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.BeanUtils;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ocip.common.IConstant;
import com.seeyon.ocip.common.entry.Address;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.exchange.api.IBussinessHandler;
import com.seeyon.ocip.exchange.exceptions.BussinessException;
import com.seeyon.ocip.exchange.model.BIZContentType;
import com.seeyon.ocip.exchange.model.BIZExchangeData;
import com.seeyon.ocip.exchange.model.BIZMessage;
import com.seeyon.ocip.exchange.model.BussinessResult;
import com.seeyon.ocip.exchange.model.Organization;
import com.seeyon.ocip.exchange.model.col.Affair;
import com.seeyon.ocip.exchange.model.col.ColObject;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import com.seeyon.ocip.exchange.model.col.ColSummary;

public class CollaborationExchangeHandler implements IBussinessHandler {
    private static final Log log = LogFactory.getLog(CollaborationExchangeHandler.class);
    private IOrganizationManager organizationManager;
    private CollaborationExchangeManager collaborationExchangeManager;
    private OrgManager orgManager;

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    public void setOrganizationManager(IOrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    @Override
    public BIZContentType type() {
        return BIZContentType.COL;
    }

    @Override
    public List<BussinessResult> exchangeReceive(BIZExchangeData in) throws BussinessException {
        List<BussinessResult> reList = new ArrayList<BussinessResult>();
        BIZMessage businessMessage = in.getBussnissMessage();
        Address sourceAddress = in.getSource();
        BIZContentType contentType = businessMessage.getContentType();
        if (BIZContentType.COL.equals(contentType)) {
            // 接收内容
            ColObject content = (ColObject) businessMessage.getContent();
            // 操作类型
            ColOperation operation = content.getOperation();
            // summary
            ColSummary colSummary = content.getColSummary();
            // 待办信息进行过滤
            List<Affair> affairs = content.getAffairs();

            // 远程的操作人员
            Address sendOrg = content.getSender();
            try {
                Long memberId = Long.parseLong(sendOrg.getId());
                String localMemberId = organizationManager.getLocalObjectId(sendOrg);
                if (Strings.isNotBlank(localMemberId)) {
                    memberId = Long.parseLong(localMemberId);
                }
                CommonUtil.ocipUserLogin(orgManager, memberId);
            } catch (Exception e) {
                if (sendOrg.getName().endsWith("管理员")) {//如果发送人是管理员则统一模拟单位管理员登录
                    List<Organization> recivers = in.getRecivers();
                    if (!recivers.isEmpty()) {
                        Organization organization = recivers.get(0);
                        String userId = organization.getIdentification().getId();//接收人平台id
                        String localMemberId = organizationManager.getLocalObjectId(IConstant.AddressType.member.name(), userId);
                        try {
                            V3xOrgMember member = orgManager.getMemberById(Long.parseLong(localMemberId));
                            CommonUtil.userLogin(orgManager, member.getOrgAccountId());
                        } catch (BusinessException e1) {
                            BussinessResult result = new BussinessResult();
                            result.setCode("2");
                            result.setMessage(e.getMessage());
                            reList.add(result);
                        }
                    }
                } else {
                    User user = new User();
                    user.setId(Long.parseLong(sendOrg.getId()));
                    user.setName(sendOrg.getName());
                    CommonUtil.setCurrentUser(user);
                }
            }
            // int opinionType = Integer.parseInt(content.getOpinionType());
            int opinionType = 1;
            String opinionContent = content.getOpinionContent();
            for (Affair affair : affairs) {
                BussinessResult result = new BussinessResult();
                result.setIdentifier(in.getIdentifier());
                Organization organization = getOrganization(in, affair);
                result.setOrganization(organization);
                // 判断是否是本系统的用户
                String userId = String.valueOf(affair.getMemberId());
                String localObjectId = organizationManager.getLocalObjectId(IConstant.AddressType.member.name(), userId);
                if (localObjectId == null) {
                    // TODO 记录日志
                    log.info("接收消息时转换本地人员信息失败");
                    result.setCode("1");
                    result.setMessage("接收消息时转换本地人员信息失败");
                    reList.add(result);
                    continue;
                }
                // 判断人员是否已经无效，则不生成相关的待办或处理业务
                try {
                    V3xOrgMember localMemberObj = orgManager.getMemberById(Long.parseLong(localObjectId));
                    if (!localMemberObj.getEnabled()) {
                        log.info("接收消息时本地人员无效");
                        result.setCode("1");
                        result.setMessage("接收消息时本地人员无效：" + localMemberObj.getName());
                        reList.add(result);
                        continue;
                    }
                } catch (Exception e) {
                    result.setCode("2");
                    result.setMessage(e.getMessage());
                }
                // 转换上一处理人
                try {
                    affair.setPreApprover(Long.valueOf(content.getSender().getId()).longValue());
                } catch (Exception e) {
                    log.error("发送协同交换信息时，转换上一处理人出错 ", e);
                }
                // 发起者和affair当前处理人是否同一人
                boolean isSelf = Long.valueOf(content.getSender().getId()).longValue() == affair.getMemberId();
                if (operation == ColOperation.FORWARD) {
                    affair.setSenderId(Long.parseLong(localObjectId));
                }
                affair.setMemberId(Long.parseLong(localObjectId));
                try {
                    // 推送类型判断
                    if (ColOperation.TODO == operation) {// 推送待办
                        collaborationExchangeManager.sendColAndMeg(affair, colSummary, new MessageReceiver(affair.getId(), affair.getMemberId().longValue(), "message.link.col.pending", new Object[]{affair.getId().toString()}), sendOrg.getName(),
                                opinionType, opinionContent, sourceAddress.toString());
                    } else if (ColOperation.DONE == operation) {// 修改Affair和ColSummary
                        collaborationExchangeManager.sendDone(affair, colSummary, sourceAddress.toString());
                    } else if (ColOperation.HASTEN == operation) {// 推送催办
                        collaborationExchangeManager.sendHasten(affair, colSummary, content.getOpinionContent(), "50".equals(content.getOpinionType()));
                    } else if (ColOperation.STOP == operation) {// 推送终止
                        collaborationExchangeManager.sendStop(affair, colSummary, sourceAddress.toString(), sendOrg.getName(), opinionContent, isSelf);
                    } else if (ColOperation.STEPBACK == operation) {// 推送回退
                        collaborationExchangeManager.sendStepBack(affair, colSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
                    } else if (ColOperation.TAKEBACK == operation) {// 推送取回
                        collaborationExchangeManager.sendStepBack(affair, colSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
                    } else if (ColOperation.COMPETE == operation) {// 推送竞争执行消息
                        collaborationExchangeManager.competitionDone(affair, colSummary);
                    } else if (ColOperation.REVOKED == operation) {// 推送撤销/回退
                        collaborationExchangeManager.sendStepBack(affair, colSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
                    } else if (ColOperation.COMMENTS == operation) {// 新增附言
                        collaborationExchangeManager.sendComments(affair, colSummary, sendOrg.getName());
                    } else if (ColOperation.OVERTIME == operation) {// 超期提醒
                        collaborationExchangeManager.sendOvertime(affair, colSummary, sendOrg.getName(),opinionContent);
                    } else if (ColOperation.MODIFYBODY == operation) {// 修改正文提醒
                        collaborationExchangeManager.sendModifyBody(affair, colSummary, sendOrg.getName());
                    } else if (ColOperation.COMMENTREPLY == operation) {// 意见回复推送消息
                        collaborationExchangeManager.sendCommentsReply(affair, colSummary, sendOrg.getName());
                    } else if (ColOperation.FAVORITE == operation) {// 收藏
                        collaborationExchangeManager.sendFavorite(affair, colSummary, sendOrg.getName());
                    } else if (ColOperation.FORWARD == operation) {// 转发
                        collaborationExchangeManager.forward(affair, colSummary, sendOrg.getName(), sourceAddress.toString());
                    } else if (ColOperation.ZCDB == operation) {// 暂存待办
                        collaborationExchangeManager.sendZcdb(affair, colSummary);
                    } else if (ColOperation.DELETE == operation) {// 删除
                        collaborationExchangeManager.fireDelete(affair, colSummary, sourceAddress.toString());
                    } else {
                        result.setCode("-1");
                        result.setMessage("操作码[" + operation.name() + "]不存在");
                    }
                } catch (Exception e) {
                    result.setCode("2");
                    result.setMessage(e.getMessage());
                    log.error("接收数据时，处理异常", e);
                }
                result.setCode("0");
                result.setMessage("协同接收成功");
                reList.add(result);
            }

        } else {
            throw new BussinessException("处理器匹配错误");
        }
        return reList;
    }

    private Organization getOrganization(BIZExchangeData in, Affair affair) {
        Organization organization = null;
        for (Organization o : in.getRecivers()) {
            if (o.getIdentification().getId().equals(affair.getMemberId().toString())) {
                organization = o;
                break;
            }
        }
        return organization;
    }

    @Override
    public BIZExchangeData exchangeSend(Map<String, Object> param) throws BussinessException {
        // 通过传输交换数据类型判断用户信息
        String localSystemCode = Global.getSysCode();
        ColOperation operation = (ColOperation) param.get("ColOperation");
        com.seeyon.apps.collaboration.po.ColSummary summary = (com.seeyon.apps.collaboration.po.ColSummary) param.get("colSummary");
        // 这里需要clone，避免hibernate自动回写影响数据
        try {
            summary = (com.seeyon.apps.collaboration.po.ColSummary) org.apache.commons.beanutils.BeanUtils.cloneBean(summary);
        } catch (Exception e) {
            log.error("summary拷贝失败", e);
        }
        AffairData data = (AffairData) param.get("affairData");
        User currentUser = new User();
        BIZExchangeData bizData = new BIZExchangeData();
        String id = UUID.randomUUID().toString().replaceAll("-", "");
        bizData.setIdentifier(id);
        // 发送方当前操作者 //TODO 可能当前是代理人员
        Organization sender = new Organization();
        V3xOrgMember currentMember = null;
        try {
            currentUser = initCurrentUser(currentUser, summary, operation);
            currentMember = orgManager.getMemberById(currentUser.getId());
            Address address = OrgUtil.getAddress(currentMember);
            sender.setIdentification(address);
            sender.setName(currentMember.getName());
        } catch (BusinessException e1) {
            throw new BussinessException("封装交换数据包失败，原因：获取当前操作者失败！", e1);
        }
        bizData.setSender(sender);

        // 接收者
        List<Affair> ocipAffairs = new ArrayList<Affair>();
        List<Organization> receivers = new ArrayList<Organization>();
        initRecivers(data, ocipAffairs, receivers);
        // 消息对象封装
        ColObject colObject = new ColObject();
        colObject.setAffairs(ocipAffairs);
        // summary
        ColSummary colSummary = new ColSummary();
        try {
            // 将本地发起人员在交换的时候替换成平台人员ID
            if (summary != null && summary.getId() != -1L) {
                OcipOrgMember sMember = OrgUtil.getPlatformMember(summary.getStartMemberId());
                if (sMember != null) {
                    summary.setStartMemberId(Long.parseLong(sMember.getId()));
                }
                initCurrentNodeInfo(localSystemCode, summary);
                bizData.setGroupId(String.valueOf(summary.getId()));
            }
        } catch (Exception e) {
            log.error("将本地发起人员在交换的时候替换成平台人员ID失败", e);
        }
        BeanUtils.convert(colSummary, summary);
        colSummary = initColSummaryByOperation(colSummary, operation, data);
        colObject.setColSummary(colSummary);
        colObject.setOperation(operation);
        colObject.setDatetime(new Date());
        // 发起者信息
        // 如果不是平台对象，需要转换成平台ID
        if (!OrgUtil.isPlatformEntity(currentMember) && Boolean.FALSE.equals(currentMember.getIsAdmin())) {
            OcipOrgMember platformMember = OrgUtil.getPlatformMember(currentMember.getId());
            // 可能当前人员在平台没有，比如系统管理员，组织管理员
            if (platformMember != null) {
                currentMember.setId(Long.valueOf(platformMember.getId()));
            }
        }
        colObject.setSender(OrgUtil.getAddress(currentMember));
        String opinionType = "0";// ?暂时没发现用处，50 发送短信
        if (data.getIsSendMessage() != null && data.getIsSendMessage()) {
            opinionType = "50";
        }
        colObject.setOpinionType(opinionType);
        Object opinionContent = param.get("opinionContent");
        colObject.setOpinionContent(opinionContent == null ? "" : opinionContent.toString());

        bizData.setRecivers(receivers);
        try {
            if (colSummary.getSubject() == null) {
                bizData.setSubject(data.getSubject());
            } else {
                String subject = ColUtil.showSubjectOfSummary(summary, false, -1, null);
                if (Strings.isBlank(subject)) {
                    subject = data.getSubject();
                }
                bizData.setSubject(subject);
            }
        } catch (Exception e) {
        }
        BIZMessage bizMessage = new BIZMessage();
        bizMessage.setContentType(BIZContentType.COL);
        bizMessage.setContent(colObject);
        // 发送源地址信息
        Address source = new Address();
        source.setId(localSystemCode);
        source.setName(localSystemCode);
        bizData.setSource(source);
        // 传送的附件
        bizMessage.setAttachments(null);
        bizData.setBussnissMessage(bizMessage);
        return bizData;
    }

    private void initCurrentNodeInfo(String localSystemCode, com.seeyon.apps.collaboration.po.ColSummary summary) {
        String currentNodesInfo = summary.getCurrentNodesInfo();
        if (Strings.isNotEmpty(currentNodesInfo)) {
            List<String> currentNodesNew = new ArrayList<String>();
            StringTokenizer tokenizer = new StringTokenizer(currentNodesInfo, ";");
            while (tokenizer.hasMoreTokens()) {
                String next = tokenizer.nextToken();
                OcipOrgMember currentMember = null;
                try {
                    currentMember = OrgUtil.getPlatformMember(Long.parseLong(next));
                } catch (Exception e) {
                }
                if (null != currentMember) {
                    currentNodesNew.add(currentMember.getId());
                } else {
                    currentNodesNew.add(next);
                }
            }
            summary.setCurrentNodesInfo(Strings.join(currentNodesNew, ";"));
        }
    }

    /**
     * 根据类型封装colSummary
     *
     * @param colSummary
     * @param operation
     * @param data
     * @return
     */
    private ColSummary initColSummaryByOperation(ColSummary colSummary, ColOperation operation, AffairData data) {
        if (operation == ColOperation.MODIFYBODY) {// 修改正文或回复意见推送
            colSummary.setCurrentNodesInfo(data.getForwardMember());// 当前处理人
            colSummary.setStartMemberId(data.getMemberId());// 当前处理人ID
        } else if (operation == ColOperation.COMMENTREPLY) {// 意见回复传送colSummary消息封装
            colSummary.setCurrentNodesInfo(data.getForwardMember());// 当前处理人
            colSummary.setStartMemberId(data.getMemberId());// 当前处理人ID
            colSummary.setCaseId(data.getCaseId());// 存储意见回复ID
            colSummary.setColType(data.getModuleType());// 意见回复commentType
            colSummary.setSubject(data.getSubject());// 回复意见的标题
            colSummary.setSuperviseTitle(data.getBodyContent());// 意见回复内容
            colSummary.setBodyType(data.getBodyType());// 意见回复接收人姓名
        } else if (operation == ColOperation.FAVORITE) {// 收藏传送colSummary消息封装
            colSummary.setState(data.getState());// 收藏类型（0，收藏 1.取消收藏）
            colSummary.setSubject(data.getSubject());// 标题
            if (data.getState() == 0) {// 新增收藏
                colSummary.setImportantLevel(data.getImportantLevel());// 借用封装appEnumKey
                colSummary.setFormid(data.getFormId());// 借用封装sourceId
                colSummary.setAutoRun(data.getIsHasAttachment());// 借用封装has
                colSummary.setResentTime(data.getResentTime());// 借用封装favoriteType
            } else if (data.getState() == 1) {// 取消收藏
                colSummary.setFormid(data.getFormId());// 借用封装sourceId
                colSummary.setCaseId(data.getCaseId());// 借用封装docId
            } else if (data.getState() == 2) {// 创建文件夹
                colSummary.setSource(data.getContentType());// 借用封装文件夹标题
                colSummary.setCaseId(data.getFormId());// 借用封装docId
            }
        }
        if (colSummary.getForwardMember() != null && !"".equals(colSummary.getForwardMember())) {
            OcipOrgMember member = OrgUtil.getPlatformMember(Long.parseLong(colSummary.getForwardMember()));
            if (null != member) {
                colSummary.setForwardMember(member.getId());
            }
        }
        return colSummary;
    }

    /**
     * 生成消息接收者
     *
     * @param data
     * @param ocipAffairs
     * @param receivers
     */
    private void initRecivers(AffairData data, List<Affair> ocipAffairs, List<Organization> receivers) {
        Map<String, Organization> receiverMap = new HashMap<String, Organization>();
        List<CtpAffair> affairs = data.getAffairList();
        if (affairs != null && affairs.size() > 0) {
            for (CtpAffair ctpAffair : affairs) {
                OcipOrgMember reciveOcipMember = OrgUtil.getPlatformMember(ctpAffair.getMemberId());
                if (reciveOcipMember == null) {
                    log.warn("处理交换数据失败，将跳过该待办数据，原因：未找到待办接收人对应的平台对象，aff = " + ctpAffair.toJSON());
                    continue;
                }
                // 转换转发人
                String forwardMemberId = ctpAffair.getForwardMember();
                if (Strings.isNotEmpty(forwardMemberId)) {
                    OcipOrgMember forwardMember = OrgUtil.getPlatformMember(forwardMemberId);
                    if (null != forwardMember) {
                        ctpAffair.setForwardMember(forwardMember.getId());
                    }
                }
                // 转换加签，会签人
                if (ctpAffair.getFormId() != null) {
                    OcipOrgMember fromMember = OrgUtil.getPlatformMember(ctpAffair.getFormId());
                    if (null != fromMember) {
                        ctpAffair.setFromId(Long.parseLong(fromMember.getId()));
                    }
                }
                Organization reciverOrg = new Organization();
                Address recAdd = new Address();
                recAdd.setId(reciveOcipMember.getId());
                recAdd.setResource(IConstant.OCIP_RESOURCE);
                recAdd.setName(reciveOcipMember.getName());
                recAdd.setType(IConstant.AddressType.member.name());
                reciverOrg.setIdentification(recAdd);
                reciverOrg.setName(reciveOcipMember.getName());
                receiverMap.put(reciveOcipMember.getId(), reciverOrg);
                ctpAffair.setMemberId(Long.parseLong(reciveOcipMember.getId()));
                OcipOrgMember sendMember = OrgUtil.getPlatformMember(ctpAffair.getSenderId());
                if (sendMember == null) {
                    ctpAffair.setSenderId(ctpAffair.getSenderId());
                } else {
                    ctpAffair.setSenderId(Long.parseLong(sendMember.getId()));
                }
                ctpAffair.setUpdateDate(new Date());
                Affair ocipAffair = initOcipAffair(ctpAffair);
                if(Strings.isBlank(ocipAffair.getSubject())) {
                	ocipAffair.setSubject(ctpAffair.getSubject());
                }
                ocipAffairs.add(ocipAffair);
            }
            if (!receiverMap.isEmpty()) {
                receivers.addAll(receiverMap.values());
            }
        }
    }

    /**
     * 查询当前发送人
     *
     * @param currentUser
     * @param operation
     * @return
     * @throws Exception
     */
    private User initCurrentUser(User currentUser, com.seeyon.apps.collaboration.po.ColSummary summary, ColOperation operation) throws BussinessException {
        if (operation != ColOperation.OVERTIME) {
            currentUser = AppContext.getCurrentUser();
        } else {
            V3xOrgMember member;
            try {
                member = orgManager.getMemberById(summary.getStartMemberId());
                log.warn("创建人员" + member.getId());
                currentUser.setId(summary.getStartMemberId());
                currentUser.setLoginAccount(member.getOrgAccountId());
                currentUser.setAccountId(member.getOrgAccountId());
                currentUser.setDepartmentId(member.getOrgDepartmentId());
                currentUser.setName(member.getName());
            } catch (BusinessException e2) {
                throw new BussinessException("通过传输交换数据类型判断用户信息,原因：获取当前操作者失败！", e2);
            }

        }
        return currentUser;
    }

    /**
     * 根据G6 affair封装 OICP affair对象
     *
     * @param ctpAffair
     * @return
     */
    private Affair initOcipAffair(CtpAffair ctpAffair) {
        Affair ocipAffair = new Affair();
        BeanUtils.convert(ocipAffair, ctpAffair);
        return ocipAffair;
    }
}
