package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.seeyon.apps.collaboration.bo.SendCollResult;
import com.seeyon.apps.collaboration.enums.ColMessageFilterEnum;
import com.seeyon.apps.collaboration.enums.CollaborationEnum;
import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.apps.collaboration.vo.ColSummaryVO;
import com.seeyon.apps.ocip.organization.bo.IOcipOrgEntity;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.common.supervise.bo.SuperviseHastenParam;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;

/**
 * OCIP协同催办切面
 */
public class CollAspectManager extends OCIPBizManagerImpl {
    private final static Log log = LogFactory.getLog(CollAspectManager.class);
    private ColManager colManager;
    private OCIPColManager ocipColManager;
    private AffairManager affairManager;
    private CollaborationExchangeManager collaborationExchangeManager;

    private EnumManager enumManagerNew;

    public void setEnumManagerNew(EnumManager enumManagerNew) {
        this.enumManagerNew = enumManagerNew;
    }

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }

    public void setOcipColManager(OCIPColManager ocipColManager) {
        this.ocipColManager = ocipColManager;
    }

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    /**
     * 催办
     *
     * @param pjd
     */
    public void onHasten(JoinPoint pjd) {
        boolean platformCall = OrgUtil.isPlatformCall();
        if (platformCall) {
            return;
        }
        SuperviseHastenParam hastenParam = (SuperviseHastenParam) pjd.getArgs()[0];
        if (!AppContext.hasPlugin("ocip")) {
            return;
        }
        // 筛选外系统人员
        long summaryId = hastenParam.getSummaryId();
        String activityId = hastenParam.getActivityId();
        ColSummary summary = null;
        List<CtpAffair> affairs = null;

        try {
            summary = colManager.getSummaryById(summaryId);
            if (summary != null && Strings.isNotEmpty(activityId)) {
                affairs = affairManager.getAffairsByObjectIdAndNodeId(summaryId, Long.parseLong(activityId));
            }
        } catch (BusinessException e) {
            log.error("根据", e);
        }
        // 收集外系统affair
        List<CtpAffair> outAffairs = new ArrayList<CtpAffair>();
        if (affairs != null) {
            //去除不在催办列表内的
            Iterator<CtpAffair> iterator = affairs.iterator();
            while (iterator.hasNext()) {
                CtpAffair affair = iterator.next();
                if (affair.getApp() != ApplicationCategoryEnum.collaboration.getKey()) {//忽略非协同事项
                    continue;
                }
                boolean has = false;
                for (Long memberId : hastenParam.getPersonIds()) {
                    if (memberId.doubleValue() == affair.getMemberId()) {
                        has = true;
                        break;
                    }
                }
                if (!has) {
                    iterator.remove();
                    continue;
                }
                if (!Integer.valueOf(StateEnum.col_pending.getKey()).equals(affair.getState())) {
                    iterator.remove();
                    continue;
                }
                outAffairs.add(affair);
            }
        }
        try {
            outAffairs = getAffairList4Exchange(outAffairs);
            if (outAffairs.isEmpty()) {
                return;
            }
        } catch (Exception e) {
            log.error("获取交换数据失败", e);
        }
        // 如果有外系统人员，推送至外系统处理
        if (!outAffairs.isEmpty()) {
            AffairData affairData = new AffairData();
            affairData.setAffairList(outAffairs);
            affairData.setIsSendMessage(true);
            affairData.setModuleId(summaryId);
            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.HASTEN, summary, hastenParam.getContent());
        }
    }

    /**
     * 跨系统转发时获取权限
     *
     * @param pjd
     * @return
     */
    public Object getPermission(ProceedingJoinPoint pjd) throws Throwable {
        if (AppContext.hasPlugin("ocip")) {
            String category = (String) pjd.getArgs()[0];
            String configItem = (String) pjd.getArgs()[1];
            Object proceed = pjd.proceed();
            if (proceed != null) {
                return proceed;
            } else {
                //如果开启OCIP 跨系统事务转发后处理时permission为空，这里默认给与所有策略，可能有安全漏洞
                return pjd.proceed(new Object[]{category, configItem, 1L});
            }
        } else {
            return pjd.proceed();
        }
    }

    /**
     * 跨系统转发
     *
     * @param transSendColl
     */
    public SendCollResult transDoForward(SendCollResult transSendColl) {
        if (AppContext.hasPlugin("ocip") && null != transSendColl && transSendColl.getSentAffair() != null) {
            StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
            for (StackTraceElement e : stackTrace) {
                if (e.getClassName().equals("com.seeyon.apps.collaboration.manager.ColManagerImpl") && e.getMethodName().equals("transDoForward")) {
                    User currentUser = AppContext.getCurrentUser();
                    if (currentUser instanceof IOcipOrgEntity) {//OCIP单点登录用户
                        AffairData data = new AffairData();
                        data.setModuleId(transSendColl.getSummary().getId());
                        data.setSender(currentUser.getId());
                        data.setSubject(transSendColl.getSummary().getSubject());
                        List<CtpAffair> list = new ArrayList<CtpAffair>();
                        list.add(transSendColl.getSentAffair());
                        data.setAffairList(list);
                        collaborationExchangeManager.handCollaborationAffair(data, ColOperation.FORWARD, transSendColl.getSummary());
                        break;
                    }
                }
            }
        }
        return transSendColl;
    }


    public void updateAffairState4RepeatAffair(JoinPoint joinPoint) {
        //在这是对自动跳过的数据进行补发
        if (AppContext.hasPlugin("ocip")) {
            CtpAffair affair = (CtpAffair) joinPoint.getArgs()[0];
            try {
                ColSummary summary = colManager.getColSummaryById(affair.getObjectId());
                AffairData ocipAffairData = new AffairData();
                ocipAffairData.setAffairList(getAffairList4Exchange(affair));
                if (ocipAffairData.getAffairList() != null && ocipAffairData.getAffairList().size() > 0 && affair.getApp() == ApplicationCategoryEnum.collaboration.getKey()) {
                    collaborationExchangeManager.handCollaborationAffair(ocipAffairData, ColOperation.TODO, summary);
                }
            } catch (Exception e1) {
                log.error("发送自动跳过外单位事务处理的出错:", e1);
            }
        }
    }

    public List<ColSummaryVO> convertCurrentNodesInfo(List<ColSummaryVO> colSummaryVOList) {
        for (ColSummaryVO vo : colSummaryVOList) {
            Long affairId = vo.getAffairId();
            try {
                CtpAffair affair = affairManager.get(affairId);
                Map reMap = ocipColManager.checkIsExchangeAffair(String.valueOf(affair.getId()), affair.getState() == StateEnum.col_sent.key() ? "2" : "0");
                if ("yes".equals(reMap.get("flag"))) {
                    Integer summaryState = affair.getSummaryState();
                    if (summaryState != null && summaryState != 1) {
                        vo.setCurrentNodesInfo("外系统事务");
                    }
                }
            } catch (Exception e) {
            }
        }
        return colSummaryVOList;
    }

    /**
     * 协同流程超期提醒
     *
     * @param joinPoint
     */
    public void dealProcessCycRemind(JoinPoint joinPoint) throws BusinessException {
        final Long objectId = (Long) joinPoint.getArgs()[0];
        final Long isAdvanceRemind = (Long) joinPoint.getArgs()[1];
        final String messageRuleId = (String) joinPoint.getArgs()[3];
        List states = new ArrayList();
        states.add(StateEnum.col_sent);
        states.add(StateEnum.col_pending);
        if (Strings.isNotBlank(messageRuleId)) {
            states.add(StateEnum.col_done);
        }
        List<CtpAffair> affairList = this.affairManager.getAffairs(objectId, states);
        if (affairList == null) {
            return;
        }
        ColSummary summary = this.colManager.getColSummaryById(objectId);
        //用于存储提醒外系统人员超期事务
        List<CtpAffair> list = new UniqueList<CtpAffair>();
        for (CtpAffair affair : affairList) {
            //判断人员是不是提醒外系统人员
            list.add(affair);
        }
        try {
            list = getAffairList4Exchange(list);
        } catch (Exception e) {
            log.error("获取交换数据失败", e);
            return;
        }
        if (list.isEmpty()) {
            return;
        }
        String forwardMemberId = "";
        int forwardMemberFlag = 0;
        String forwardMember = null;
        String msgKey = null;
        Integer messageFilterParam = ColMessageFilterEnum.overTime.key;
        if (Strings.isNotBlank(forwardMemberId)) {
            try {
                forwardMember = getOrgManager().getMemberById(Long.parseLong(forwardMemberId))
                        .getName();
                forwardMemberFlag = 1;
            } catch (Exception e) {
                log.error("", e);
            }
        }
        if (affairList.size() > 0) {
            forwardMemberId = ((CtpAffair) affairList.get(0)).getForwardMember();
        }

        if (isAdvanceRemind == 0l) {
            msgKey = "process.summary.advanceRemind";
        } else {
            msgKey = "process.summary.overTerm";
            int autoStopRepealFlag;
            if (summary.getCanAutostopflow()) {
                autoStopRepealFlag = CollaborationEnum.processTermType.autoStop.ordinal();
            } else {
                autoStopRepealFlag = (summary.getProcessTermType() == null) ? -1 : summary.getProcessTermType().intValue();
            }
            if (autoStopRepealFlag == CollaborationEnum.processTermType.autoStop.ordinal() || autoStopRepealFlag == CollaborationEnum.processTermType.autoRepeal.ordinal()) {
                return;
            }
        }
        if (list != null && list.size() > 0) {
            Long advanceRemind = summary.getAdvanceRemind();
            CtpEnumItem cei = this.enumManagerNew.getEnumItem(EnumNameEnum.common_remind_time, String.valueOf(advanceRemind == null ? 0L : advanceRemind));
            String enumLabel = ResourceUtil.getString(cei.getLabel());
            AffairData affairData = new AffairData();
            affairData.setAffairList(list);
            affairData.setIsSendMessage(false);
            affairData.setSubject(summary.getSubject());
            affairData.setModuleId(objectId);
            //借用colSummary组装传输提醒数据
            affairData.setSubject(summary.getSubject());//title
            affairData.setContentType(msgKey);//msgKey
            affairData.setResentTime(forwardMemberFlag);//forwardMemberFlag
            affairData.setForwardMember(forwardMember);//forwardMember
            affairData.setSender(summary.getStartMemberId());//sendId
            affairData.setState(messageFilterParam);//messageFilterParam
            affairData.setImportantLevel(summary.getImportantLevel());
            //要将msgKey，title，forwardMemberFlag，forwardMember，enumLabel，sendId，messageFilterParam，appEnum
            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.OVERTIME, summary, msgKey + "|" + enumLabel);
        }
    }

    /**
     * 协同节点超期提醒
     * TODO
     *
     * @param joinPoint
     */
    public void transOverTimeProcessing(JoinPoint joinPoint) {
        final Long affairId = (Long) joinPoint.getArgs()[0];
        final Long isAdvanceRemind = (Long) joinPoint.getArgs()[1];
       // final String messageRuleId = (String) joinPoint.getArgs()[3];
        CtpAffair affair = null;
        try {
            affair = affairManager.get(affairId);
        } catch (BusinessException e2) {
            log.error(e2.getMessage(), e2);
        }
        if (null == affair || affair.getCompleteTime() != null) {
            return;
        }
        Long objectId = affair.getObjectId();
        try {
            ColSummary summary = this.colManager.getColSummaryById(objectId);
            //用于存储提醒外系统人员超期事务
            List<CtpAffair> list = new UniqueList<CtpAffair>();
            list.add(affair);
            list = getAffairList4Exchange(list);
            if (list.isEmpty()) {
                return;
            }
            String forwardMemberId = affair.getForwardMember();
            int forwardMemberFlag = 0;
            String forwardMember = null;
            String msgKey = null;
            Integer messageFilterParam = ColMessageFilterEnum.overTime.key;
            if (Strings.isNotBlank(forwardMemberId)) {
                try {
                    forwardMember = getOrgManager().getMemberById(Long.parseLong(forwardMemberId))
                            .getName();
                    forwardMemberFlag = 1;
                } catch (Exception e) {
                    log.error("", e);
                }
            }

            if (isAdvanceRemind.equals(0l)) {
                msgKey = "process.summary.advanceRemind";
            } else {
                msgKey = "process.summary.overTerm";
                int autoStopRepealFlag;
                if (summary.getCanAutostopflow()) {
                    autoStopRepealFlag = CollaborationEnum.processTermType.autoStop.ordinal();
                } else {
                    autoStopRepealFlag = (summary.getProcessTermType() == null) ? -1 : summary.getProcessTermType().intValue();
                }
                if (autoStopRepealFlag == CollaborationEnum.processTermType.autoStop.ordinal() || autoStopRepealFlag == CollaborationEnum.processTermType.autoRepeal.ordinal()) {
                    return;
                }
            }

            Long advanceRemind = summary.getAdvanceRemind();
            CtpEnumItem cei = this.enumManagerNew.getEnumItem(EnumNameEnum.common_remind_time, String.valueOf(advanceRemind == null ? 0L : advanceRemind));
            String enumLabel = ResourceUtil.getString(cei.getLabel());
            AffairData affairData = new AffairData();
            affairData.setAffairList(list);
            affairData.setIsSendMessage(false);
            affairData.setSubject(summary.getSubject());
            affairData.setModuleId(objectId);
            //借用colSummary组装传输提醒数据
            affairData.setSubject(summary.getSubject());//title
            affairData.setContentType(msgKey);//msgKey
            affairData.setResentTime(forwardMemberFlag);//forwardMemberFlag
            affairData.setForwardMember(forwardMember);//forwardMember
            affairData.setSender(summary.getStartMemberId());//sendId
            affairData.setState(messageFilterParam);//messageFilterParam
            affairData.setImportantLevel(summary.getImportantLevel());
            //要将msgKey，title，forwardMemberFlag，forwardMember，enumLabel，sendId，messageFilterParam，appEnum
            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.OVERTIME, summary, msgKey + "|" + enumLabel);
        } catch (Exception e) {
           // e.printStackTrace();
            log.error("推送节点超期消息异常", e);

        }

    }
}
