package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.seeyon.apps.agent.bo.MemberAgentBean;
import com.seeyon.apps.collaboration.dao.ColDao;
import com.seeyon.apps.collaboration.enums.ColMessageFilterEnum;
import com.seeyon.apps.collaboration.enums.CollaborationEnum;
import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.manager.ColMessageManager;
import com.seeyon.apps.collaboration.util.ColUtil;
import com.seeyon.apps.doc.api.DocApi;
import com.seeyon.apps.doc.bo.DocResourceBO;
import com.seeyon.apps.doc.enums.DocActionEnum;
import com.seeyon.apps.ocip.util.CommonUtil;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.usermessage.MessageContent;
import com.seeyon.ctp.common.usermessage.MessageReceiver;
import com.seeyon.ctp.common.usermessage.UserMessageManager;
import com.seeyon.ctp.common.usermessage.UserMessageUtil;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.BeanUtils;
import com.seeyon.ctp.util.CommonTools;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.ocip.common.IConstant;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.configuration.OcipConfiguration;
import com.seeyon.ocip.exchange.api.IBussinessService;
import com.seeyon.ocip.exchange.exceptions.ExchangeException;
import com.seeyon.ocip.exchange.model.BIZContentType;
import com.seeyon.ocip.exchange.model.col.Affair;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import com.seeyon.ocip.exchange.model.col.ColSummary;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CollaborationExchangeManagerImpl implements CollaborationExchangeManager {
    private static final Log log = LogFactory.getLog(CollaborationExchangeManagerImpl.class);
    private UserMessageManager userMessageManager;
    private WorkflowApiManager wapi;
    private ColManager colManager;
    private AffairManager affairManager;
    private ColDao colDao;
    private IOrganizationManager organizationManager;
    private OrgManager orgManager;
    private DocApi docApi;
    private ColMessageManager colMessageManager;

    public void setColMessageManager(ColMessageManager colMessageManager) {
        this.colMessageManager = colMessageManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setOrganizationManager(IOrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    public void setColDao(ColDao colDao) {
        this.colDao = colDao;
    }

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }


    public void setUserMessageManager(UserMessageManager userMessageManager) {
        this.userMessageManager = userMessageManager;
    }

    private IBussinessService getBussinessService() {
        return OcipConfiguration.getInstance().getExchangeSpi().getBussinessService();
    }

    @Override
    public void handCollaborationAffair(AffairData data, ColOperation operation, com.seeyon.apps.collaboration.po.ColSummary colSummary) {
        handCollaborationAffair(data, operation, colSummary, null);
    }

    @Override
    public void handCollaborationAffair(AffairData data, ColOperation operation, com.seeyon.apps.collaboration.po.ColSummary colSummary, String opinionContent) {
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("ColOperation", operation);
        param.put("affairData", data);
        param.put("colSummary", colSummary);
        if (opinionContent != null) {
            param.put("opinionContent", opinionContent);
        }
        try {
            getBussinessService().fireExchange(BIZContentType.COL, param);
        } catch (ExchangeException e) {
            log.error("发送数据失败", e);
        }
    }

    /**
     * 回退
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendStepBack(Affair affair, ColSummary colSummary, String sendName, ColOperation operation, String opinionContent, boolean isSelf, String systemCode) {
        try {
            //保存待办和colSummary
            CtpAffair v5Affair = new CtpAffair();
            BeanUtils.convert(v5Affair, affair);
            if (v5Affair.getApp() == null) {
                v5Affair.setApp(ApplicationCategoryEnum.collaboration.ordinal());
            }
            com.seeyon.apps.collaboration.po.ColSummary v5Summary = colManager.getSummaryById(v5Affair.getObjectId());

            //发送消息
            Long msgSenderId = v5Affair.getSenderId();
            String froward = v5Affair.getForwardMember();
            int isFroward = Strings.isBlank(froward) ? 0 : 1;
            MessageReceiver receiver = new MessageReceiver(affair.getId(), affair.getMemberId().longValue(),
                    "message.link.col.pending", new Object[]{affair.getId().toString()});
            v5Affair.setSubState(SubStateEnum.col_pending_read.key());
            Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
            paraMap.put("ocipSystemCode", systemCode);
            AffairUtil.setExtProperty(v5Affair, paraMap);
            //v5Affair.setAddition(systemCode);
            String messageLink = "collaboration.msg.takeback";
            //特殊判断-对于回退回来的消息接收人包含自己的要过滤掉
            try {
                String senderIdStr = organizationManager.getLocalObjectId(IConstant.AddressType.member.name(), String.valueOf(affair.getSenderId()));
                if (senderIdStr != null && Long.parseLong(senderIdStr) == affair.getMemberId()) {
                    isSelf = true;
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
            switch (operation) {
                case STEPBACK:
                    v5Affair.setState(StateEnum.col_stepBack.getKey());
                    convertBackFromId(v5Affair);
                    if (!isSelf) {//控制是否发送消息
                        userMessageManager.sendSystemMessage(new MessageContent("collaboration.message.stepBack", affair.getSubject(), sendName, isFroward,
                                        froward, opinionContent, -1).setImportantLevel(affair.getImportantLevel()),
                                ApplicationCategoryEnum.collaboration, msgSenderId, receiver, v5Affair.getImportantLevel());
                    }
                    break;
                case TAKEBACK:
                    //取回的时候回发送两条TAKEBACK包，一条已办数据，另一条为下一个节点的事项数据，这里暂时以状态区分:待办状态为主动取回的事项，撤销状态的为下一个节点的事项数据
//                    CtpAffair localAffair = affairManager.get(v5Affair.getId());
                    if (Integer.valueOf(StateEnum.col_pending.key()).equals(v5Affair.getState())) {
                        v5Affair.setSubState(SubStateEnum.col_pending_unRead.key());
                        v5Affair.setCompleteTime(null);
                        v5Affair.setOverTime(null);
                        v5Affair.setOverWorktime(null);
                        v5Affair.setRunTime(null);
                        v5Affair.setRunWorktime(null);
                        // 设置代理人
                        Long proxyMemberId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.getKey(), v5Affair.getMemberId(), colSummary.getTempleteId());
                        v5Affair.setProxyMemberId(proxyMemberId);
                    }
                    if (!isSelf) {//控制是否发送消息
                        messageLink = "collaboration.msg.takeback";
                        MessageContent content1 = MessageContent.get(messageLink, affair.getSubject(), sendName, isFroward, froward);
                        userMessageManager.sendSystemMessage(content1, ApplicationCategoryEnum.collaboration, msgSenderId, receiver);
                    }
                    break;
                case REVOKED:
                    v5Affair.setState(StateEnum.col_cancel.getKey());
                    convertBackFromId(v5Affair);
                    if (!isSelf) {//控制是否发送消息
                        messageLink = "collaboration.summary.cancel";
                        userMessageManager.sendSystemMessage(new MessageContent(messageLink, v5Affair.getSubject(), sendName, opinionContent, isFroward, froward, 1)
                                .setImportantLevel(v5Affair.getImportantLevel()), ApplicationCategoryEnum.collaboration, v5Affair.getSenderId(), receiver, v5Affair);
                    }
                    break;

            }

            this.saveOrUpdateSummaryAndAffair(null, v5Affair);

            if (v5Affair.getDeadlineDate() != null && v5Affair.getDeadlineDate() != 0) {
                ColUtil.deleteQuartzJobForNode(v5Affair);
            }
            //affairManager.updateAffairsState2Cancel(v5Summary.getId());  
            v5Summary.setCaseId(null);
            v5Summary.setState(CollaborationEnum.flowState.cancel.ordinal());
            v5Summary.setCoverTime(Boolean.FALSE);
            //更新当前处理人信息
            ColUtil.updateCurrentNodesInfo(v5Summary);

            //edocStatManager.deleteEdocStat(v5Summary.getId());
            this.saveOrUpdateSummaryAndAffair(v5Summary, null);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将非本地回退人id转化为平台人员id
     *
     * @param v5Affair
     */
    private void convertBackFromId(CtpAffair v5Affair) {
        Long backFromId = v5Affair.getBackFromId();
        if (backFromId != null) {//设置撤销/回退人员id
            try {
                V3xOrgMember dMember = orgManager.getMemberById(backFromId);
                if (OrgUtil.isPlatformEntity(dMember)) {//回退人为平台人员
                    //OcipOrgMember platformMember = OrgUtil.getPlatformMember(backFromId);
                    v5Affair.setBackFromId(dMember.getId());
                }
            } catch (BusinessException e) {

            }
        }
    }

    /**
     * 推送待办并发送消息
     *
     * @param affair
     * @param colSummary
     * @param receiver
     */
    @Override
    public void sendColAndMeg(Affair affair, ColSummary colSummary, MessageReceiver receiver, String sendName, int opinionType, String opinionContent, String sendSysCode) {

        //保存待办和colSummary
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        if (affair.getFromId() == null) {
            v5Affair.setPreApprover(affair.getSenderId());
        }
        // 设置代理人
        Long proxyMemberId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.getKey(), v5Affair.getMemberId(), colSummary.getTempleteId());
        v5Affair.setProxyMemberId(proxyMemberId);

        //v5Affair.setAddition(sendSysCode);
        Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
        paraMap.put("ocipSystemCode", sendSysCode);
        AffairUtil.setExtProperty(v5Affair, paraMap);
        this.saveOrUpdateSummaryAndAffair(v5Summary, v5Affair);
        //affairManager.updateAffair(v5Affair);
        //发送消息
        String froward = v5Affair.getForwardMember();
        int isFroward = Strings.isBlank(froward) ? 0 : 1;
        String forwardName = "";
        if (isFroward == 1) {
            OcipOrgMember forwardMember = organizationManager.getMember(froward);
            if (null != forwardMember) {
                forwardName = forwardMember.getName();
            }
        }
        Long msgSenderId = v5Affair.getSenderId();

        Object[] subjects = new Object[]{affair.getSubject(), sendName, isFroward, forwardName, 0, "", 0};
        MessageContent mContent = MessageContent.get("col.send", subjects);
        try {
            userMessageManager.sendSystemMessage(
                    mContent.setImportantLevel(v5Affair.getImportantLevel()),
                    ApplicationCategoryEnum.collaboration, msgSenderId, receiver, v5Affair.getImportantLevel());
            //MessageContent mc = new MessageContent("collaboration.opinion.deal", sendName, affair.getSubject(), 0, null, opinionType, opinionContent, -1, -1, 0);
            //userMessageManager.sendSystemMessage(mc, ApplicationCategoryEnum.collaboration,msgSenderId, receiver);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 催办
     *
     * @param affair
     * @param colSummary
     * @param msgContent
     * @param sendMsg
     */
    @Override
    public void sendHasten(Affair affair, ColSummary colSummary, String msgContent, boolean sendMsg) {
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        this.saveOrUpdateSummaryAndAffair(v5Summary, null);
        String processId = colSummary.getProcessId();
        String activityId = String.valueOf(affair.getActivityId());
        List<String> receviers = new ArrayList<String>();
        receviers.add(String.valueOf(affair.getMemberId()));
        User user = AppContext.getCurrentUser();
        if (user == null) {
            user = new User();
            Long userId = affair.getSenderId();
            user.setId(userId);
            OcipOrgMember member = organizationManager.getMember(userId.toString());
            if (null != member) {
                user.setName(member.getName());
            }
            CommonUtil.setCurrentUser(user);
        }

        OrgUtil.setIsPlatformCall();
        wapi.hasten(processId, activityId, "collaboration", receviers, null, msgContent, sendMsg);
        //workFlowHastenManager.hasten(processId, activityId, "collaboration", receviers, null, msgContent, sendMsg);
    }

    /**
     * 已办
     *
     * @param affair
     * @param colSummary
     * @param sendSysCode
     */
    @Override
    public void sendDone(Affair affair, ColSummary colSummary, String sendSysCode) {
        //保存待办和colSummary
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        //v5Affair.setAddition(sendSysCode);
        Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
        paraMap.put("ocipSystemCode", sendSysCode);
        AffairUtil.setExtProperty(v5Affair, paraMap);
        com.seeyon.apps.collaboration.po.ColSummary oldSummary = colDao.getColSummaryById(v5Summary.getId());
        if (oldSummary.getFinishDate() != null) {
            v5Summary.setState(CollaborationEnum.flowState.finish.ordinal());//已完成
            v5Summary.setCurrentNodesInfo("");
            v5Summary.setFinishDate(oldSummary.getFinishDate());
        }
        //对于没有传输完成时间的补起
        if (v5Summary.getFinishDate() != null && v5Affair.getCompleteTime() == null) {
            v5Summary.setState(CollaborationEnum.flowState.finish.ordinal());//已完成
            v5Summary.setCurrentNodesInfo("");
            v5Affair.setCompleteTime(new Date());
        }
        this.saveOrUpdateSummaryAndAffair(v5Summary, v5Affair);

        if (affair.getTransactorId() != null) {//代理人处理
            try {
                String forwardMemberName = getForwardMemberName(affair.getForwardMember());
                V3xOrgMember member = orgManager.getMemberById(affair.getMemberId());
                String proxyName = orgManager.getMemberById(affair.getTransactorId()).getName();
                MessageContent mc = new MessageContent("collaboration.opinion.deal", member.getName(),
                        affair.getSubject(), forwardMemberName == null ? 0 : 1, forwardMemberName, 1, "", 0, -1, 0);
                mc.setImportantLevel(affair.getImportantLevel());
                mc.add("col.agent.deal", proxyName);

                Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
                MessageReceiver receiver = new MessageReceiver(affair.getId(), affair.getMemberId(), "message.link.col.done.detail", affair.getId(), null);
                receiver.setReply(true);
                receivers.add(receiver);

                userMessageManager.sendSystemMessage(mc, ApplicationCategoryEnum.collaboration, affair.getMemberId(), receivers, ColUtil.getImportantLevel(v5Affair));
            } catch (Exception e) {
                log.error("发送已办消息失败", e);
            }
        }
    }

    /**
     * 终止
     *
     * @param affair
     * @param colSummary
     * @param sendSysCode
     * @param sendName
     * @param opinionContent
     * @param isSelf
     */
    @Override
    public void sendStop(Affair affair, ColSummary colSummary, String sendSysCode, String sendName, String opinionContent, boolean isSelf) {

        affair.setSummaryState(CollaborationEnum.flowState.terminate.ordinal());
        //保存待办和colSummary
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        //v5Affair.setAddition(sendSysCode);
        Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
        paraMap.put("ocipSystemCode", sendSysCode);
        AffairUtil.setExtProperty(v5Affair, paraMap);
        this.saveOrUpdateSummaryAndAffair(v5Summary, v5Affair, ColOperation.STOP);

//        try {
//            v5Affair.setSummaryState(CollaborationEnum.flowState.terminate.ordinal());
//            affairManager.updateAffair(v5Affair);
//        } catch (BusinessException e) {
//            log.error("更新跨系统事务终止状态异常",e);
//        }

        //发送提醒信息
        String froward = v5Affair.getForwardMember();
        int isFroward = Strings.isBlank(froward) ? 0 : 1;
        Long msgSenderId = v5Affair.getSenderId();
        Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
        receivers.add(new MessageReceiver(affair.getId(), affair.getMemberId(), "message.link.col.done.detail", affair.getId()));

        if (!isSelf) {//控制是否发送消息
            try {
                userMessageManager.sendSystemMessage(new MessageContent("col.terminate", v5Affair.getSubject(),
                                sendName, isFroward, froward, -1, opinionContent).setImportantLevel(v5Affair.getImportantLevel()),
                        ApplicationCategoryEnum.collaboration, msgSenderId, receivers, v5Affair.getImportantLevel());
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }

    }

    /**
     * 新增附言
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendComments(Affair affair, ColSummary colSummary, String sendName) {
        //保存待办和colSummary
        Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
        String opinionContent = UserMessageUtil.getComment4Message(Strings.toText(affair.getSubject()));
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        Long memberId = v5Affair.getMemberId();
        if (v5Affair.getState() == StateEnum.col_pending.key()) {
            receivers.add(new MessageReceiver(affair.getId(), memberId, "message.link.col.pending", affair.getId(), affair.getTempleteId()));
        } else {
            receivers.add(new MessageReceiver(affair.getId(), memberId, "message.link.col.done.detail", affair.getId(), affair.getTempleteId()));
        }
        Integer importantLevel = ColUtil.getImportantLevel(v5Affair);
        String forwardMemberName = getForwardMemberName(v5Affair.getForwardMember());
        try {
            userMessageManager.sendSystemMessage(new MessageContent("col.addnote", affair.getAddition(), sendName, forwardMemberName == null ? 0 : 1, forwardMemberName, opinionContent).setImportantLevel(v5Affair.getImportantLevel()),
                    ApplicationCategoryEnum.collaboration, v5Affair.getSenderId(), receivers, importantLevel);
        } catch (BusinessException e) {
            throw new RuntimeException("协同交换发起人增加附言消息提醒失败", e);
        }
    }

    /**
     * 超期提醒
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendOvertime(Affair affair, ColSummary colSummary, String sendName, String opinionContent) {
        //保存待办和colSummary
        Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        Long memberId = v5Affair.getMemberId();
        if (v5Affair.getState() == StateEnum.col_pending.key()) {
            receivers.add(new MessageReceiver(affair.getId(), memberId, "message.link.col.pending", affair.getId().toString()));
        } else {
            receivers.add(new MessageReceiver(affair.getId(), memberId, "message.link.col.done.detail", affair.getId().toString()));
        }
        try {
            String msgKey = String.valueOf(opinionContent).contains("advanceRemind") ? "process.summary.advanceRemind" : "process.summary.overTerm";
            String[] strings = String.valueOf(opinionContent).split("\\|");
            String enumLabel = "";
            if (strings.length > 1) {
                enumLabel = strings[1];
            }
            String forwardMember = colSummary.getForwardMember();
            MessageContent content = MessageContent.get(msgKey, new Object[]{colSummary.getSubject(), Strings.isBlank(forwardMember) ? 0 : 1, forwardMember, enumLabel}).setImportantLevel(v5Affair.getImportantLevel());
            userMessageManager.sendSystemMessage(content, ApplicationCategoryEnum.collaboration, v5Affair.getSenderId(), receivers, new Object[]{ColMessageFilterEnum.overTime.key});
        } catch (BusinessException e) {
            throw new RuntimeException("协同交换流程超期发送消息提醒失败", e);
        }
    }

    /**
     * 修改正文提醒
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendModifyBody(Affair affair, ColSummary colSummary, String sendName) {
        //保存待办和colSummary
        Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        String messageUrl = "";
        if (Integer.valueOf(StateEnum.col_pending.getKey()).equals(affair.getState())) {
            messageUrl = "message.link.col.pending";
        } else if (Integer.valueOf(StateEnum.col_sent.getKey()).equals(affair.getState())) {
            messageUrl = "message.link.col.waiSend";
        } else {
            messageUrl = "message.link.col.done.detail";
        }
        String forwardMemberName = getForwardMemberName(v5Affair.getForwardMember());
        if (affair.getState() == -100) {//推送消息给发起人
            Long sendId = affair.getSenderId();
            receivers.add(new MessageReceiver(affair.getId(), sendId, messageUrl, affair.getId().toString(), ""));
        } else {
            receivers.add(new MessageReceiver(affair.getId(), affair.getMemberId(), "message.link.col.pending", affair.getId().toString(), ""));
        }
        try {
            //发消息
            MessageContent messageContent = new MessageContent("col.modifyBodyOrAtt", colSummary.getSubject(), colSummary.getCurrentNodesInfo(), forwardMemberName == null ? 0 : 1, forwardMemberName, colSummary.getColType());
            messageContent.setImportantLevel(v5Affair.getImportantLevel());
            userMessageManager.sendSystemMessage(messageContent, ApplicationCategoryEnum.collaboration, AppContext.getCurrentUser().getId(), receivers, v5Affair.getImportantLevel());
        } catch (BusinessException e) {
            throw new RuntimeException("协同交换流程发送消息提醒失败", e);
        }
    }

    /**
     * 意见回复
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendCommentsReply(Affair affair, ColSummary colSummary, String sendName) {
        //保存待办和colSummary
        Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        String messageUrl = "";
        if (Integer.valueOf(StateEnum.col_pending.getKey()).equals(v5Affair.getState())) {
            messageUrl = "message.link.col.pending";
        } else {
            messageUrl = "message.link.col.done.detail";
        }
        String froward = v5Affair.getForwardMember();
        int isFroward = Strings.isBlank(froward) ? 0 : 1;
        receivers.add(new MessageReceiver(affair.getId(), affair.getMemberId(), messageUrl, affair.getId().toString(), colSummary.getCaseId()));
        try {
            //发消息
            userMessageManager.sendSystemMessage(new MessageContent("col.reply", colSummary.getSubject(), colSummary.getCurrentNodesInfo(), colSummary.getBodyType(),
                            isFroward, froward, colSummary.getColType(), colSummary.getSuperviseTitle()).setImportantLevel(v5Affair.getImportantLevel()),
                    ApplicationCategoryEnum.collaboration, colSummary.getStartMemberId(), receivers, v5Affair.getImportantLevel());
        } catch (BusinessException e) {
            throw new RuntimeException("协同交换流程发送消息提醒失败", e);
        }
    }

    @Override
    public void competitionDone(Affair affair, ColSummary colSummary) {
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        User user = AppContext.getCurrentUser();
        if (user == null) {
            user = new User();
            //发送端将senderId替换为处理者
            Long userId = affair.getSenderId();
            user.setId(userId);
            try {
                V3xOrgMember member = orgManager.getMemberById(userId);
                if (null != member) {
                    user.setName(member.getName());
                }
                CommonUtil.setCurrentUser(user);
            } catch (BusinessException e) {
                log.error(e.getMessage(), e);
            }
        }
        List<CtpAffair> affairs = null;
        try {
            affairs = affairManager.getAffairsByObjectIdAndNodeId(affair.getObjectId(), affair.getActivityId());
        } catch (BusinessException e) {
            log.error(e.getMessage(), e);
        }
        if (null != affairs && !affairs.isEmpty()) {
            StringBuffer hql = new StringBuffer();
            hql.append("update CtpAffair set state=:state,subState=:subState,updateDate=:updateDate where id=:id");
            Map<String, Object> params = new HashMap<String, Object>();
            params.put("state", StateEnum.col_competeOver.key());
            params.put("subState", SubStateEnum.col_normal.key());
            params.put("updateDate", new Date());
            params.put("id", affair.getId());
            try {
                affairManager.update(hql.toString(), params);
            } catch (BusinessException e) {
                log.error(e.getMessage(), e);
            }

            // 给在竞争执行中被取消的affair发送消息提醒
            transCompetitionCancel(v5Affair);
        }
        // 更新当前处理人信息
        com.seeyon.apps.collaboration.po.ColSummary summary = null;
        try {
            summary = colManager.getColSummaryById(affair.getObjectId());
        } catch (BusinessException e) {
            log.error(e.getMessage(), e);
        }
        if (summary == null) {
            return;
        }
        if (affair.getState().equals(StateEnum.col_pending.getKey())) {// 当前节点为待办的时候增加到当前处待办人信息
            summary.setCurrentNodesInfo(affair.getMemberId() + "");
        } else {
            summary.setCurrentNodesInfo("");
        }
    }

    //给在竞争执行中被取消的affair发送消息提醒
    public Boolean transCompetitionCancel(CtpAffair ctpAffair) {
        Integer importantLevel = ColUtil.getImportantLevel(ctpAffair);
        User user = AppContext.getCurrentUser();
        String userName = "";
        Long userId = null;
        if (user != null) {
            userName = user.getName();
            userId = user.getId();
        }
        if (userId == null || Strings.isBlank(userName)) {
            return false;
        }
        String forwardMemberId = ctpAffair.getForwardMember();
        int forwardMemberFlag = 0;
        String forwardMember = null;
        try {
            if (Strings.isNotBlank(forwardMemberId)) {
                forwardMember = orgManager.getMemberById(Long.parseLong(forwardMemberId)).getName();
                forwardMemberFlag = 1;
            }
            //节点中的人员
            Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
            //代理人集合
            Set<MessageReceiver> receivers1 = new HashSet<MessageReceiver>();

            List<Long> memberIds = new ArrayList<Long>();
            //不用删除定时任务了，定时任务里面已经做了判断，被竞争掉的不执行了
            //获取代理人，发消息时给代理发
//                Long agentId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.key(),ctpAffair.getMemberId());
//                if(agentId!=null && !userId.equals(agentId)){
//                    //代理人不在节点中，则发送消息的时候添加代理人
//                    if (!memberIds.contains(ctpAffair.getMemberId())) {
//                        receivers1.add(new MessageReceiver(ctpAffair.getId(), agentId));
//                        memberIds.add(ctpAffair.getMemberId());
//                    }
//                }
            if (!ctpAffair.getMemberId().equals(userId) && !memberIds.contains(ctpAffair.getMemberId())) {
                receivers.add(new MessageReceiver(ctpAffair.getId(), ctpAffair.getMemberId()));
                memberIds.add(ctpAffair.getMemberId());
            }

            V3xOrgMember member = null;
            //是否是代理人
            boolean hasProxy = false;
//            //判断是否是代理人
//            Long agentId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.key(),ctpAffair.getMemberId());
//            if (!userId.equals(ctpAffair.getMemberId()) && userId.equals(agentId)) {
//                hasProxy = true;
//                member = getMemberById(orgManager, ctpAffair.getMemberId());
//            }
            //代理人处理发送消息
            if (hasProxy) {
                String proxyName = member.getName();
                userMessageManager.sendSystemMessage(new MessageContent("col.competition", ctpAffair.getSubject(),
                        proxyName, forwardMemberFlag, forwardMember).add("col.agent.deal", user.getName()).setImportantLevel(ctpAffair.getImportantLevel()), ApplicationCategoryEnum.collaboration, member.getId(), receivers, importantLevel);
                //给代理人发消息
                if (receivers1 != null && receivers1.size() != 0) {
                    userMessageManager.sendSystemMessage(new MessageContent("col.competition", ctpAffair.getSubject(),
                                    proxyName, forwardMemberFlag, forwardMember).add("col.agent.deal", user.getName())
                                    .add("col.agent").setImportantLevel(ctpAffair.getImportantLevel()),
                            ApplicationCategoryEnum.collaboration, member.getId(), receivers1, importantLevel);
                }
            } else {
                userMessageManager.sendSystemMessage(new MessageContent("col.competition", ctpAffair.getSubject(),
                        userName, forwardMemberFlag, forwardMember).setImportantLevel(ctpAffair.getImportantLevel()), ApplicationCategoryEnum.collaboration, ctpAffair.getSenderId(), receivers, importantLevel);
                //给代理人发消息
                if (receivers1 != null && receivers1.size() != 0) {
                    userMessageManager.sendSystemMessage(new MessageContent("col.competition", ctpAffair.getSubject(),
                                    userName, forwardMemberFlag, forwardMember)
                                    .add("col.agent").setImportantLevel(ctpAffair.getImportantLevel()),
                            ApplicationCategoryEnum.collaboration, ctpAffair.getSenderId(), receivers1, importantLevel);
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return true;
    }

    /**
     * 收藏
     *
     * @param affair
     * @param colSummary
     * @param sendName
     */
    @Override
    public void sendFavorite(Affair affair, ColSummary colSummary, String sendName) {
        try {
            V3xOrgMember dealUser = orgManager.getMemberById(affair.getMemberId());
            if (colSummary.getState() == 0) {//收藏操作
                docApi.saveCollaborationExchangeFavorite(affair.getMemberId(), colSummary.getImportantLevel(), colSummary.getFormid(), colSummary.getAutoRun(), colSummary.getResentTime(), dealUser.getOrgAccountId());
            } else if (colSummary.getState() == 1) {//取消收藏操作
            	docApi.cancelCollaborationExchangeFavorite(colSummary.getCaseId(), colSummary.getFormid(), affair.getMemberId());
            } else if (colSummary.getState() == 2) {//修建文件夹
                docApi.createCollFavoriteFloder(colSummary.getSource(), colSummary.getCaseId(), affair.getMemberId());
            }
            /*//保存待办和colSummary
            Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
			CtpAffair v5Affair = new CtpAffair();
	    	BeanUtils.convert(v5Affair, affair);
	        String messageUrl = "";
	        if (Integer.valueOf(StateEnum.col_pending.getKey()).equals(v5Affair.getState())) {
                messageUrl = "message.link.col.pending";
            } else {
                messageUrl = "message.link.col.done.detail";
            }
            receivers.add(new MessageReceiver(affair.getId(), affair.getMemberId(), messageUrl,  affair.getId().toString()));
	        try {
	        	if(colSummary.getState() == 0){//收藏操作
	                userMessageManager.sendSystemMessage(new MessageContent("col.favorite.add",  colSummary.getSubject()).setImportantLevel(v5Affair.getImportantLevel()),
	                        ApplicationCategoryEnum.collaboration, v5Affair.getSenderId(), receivers,  v5Affair.getImportantLevel());
				}else{//取消收藏操作
	                userMessageManager.sendSystemMessage(new MessageContent("col.favorite.del",  colSummary.getSubject()).setImportantLevel(v5Affair.getImportantLevel()),
	                        ApplicationCategoryEnum.collaboration, v5Affair.getSenderId(), receivers,  v5Affair.getImportantLevel());
				}		
	        	
	        } catch (MessageException e) {
	        	log.error("协同交换收藏功能发送消息提醒失败", e);
	            throw new BusinessException("sendMessage Failed");
	        }	        */
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    protected void saveOrUpdateSummaryAndAffair(com.seeyon.apps.collaboration.po.ColSummary v5Summary, CtpAffair v5Affair) {
        saveOrUpdateSummaryAndAffair(v5Summary, v5Affair, null);
    }

    /**
     * 跨系统交换收藏操作
     *
     * @param userId
     * @param appEnumKey
     * @param sourceId
     * @param isHasAtt
     * @param favoriteType
     */

    /**
     * 统一处理事务的更新
     *
     * @param v5Summary
     * @param v5Affair
     */
    protected void saveOrUpdateSummaryAndAffair(com.seeyon.apps.collaboration.po.ColSummary v5Summary, CtpAffair v5Affair,ColOperation operation) {
        try {
            if (v5Summary != null) {
                Long summaryId = v5Summary.getId();
                com.seeyon.apps.collaboration.po.ColSummary oldSummary = colManager.getSummaryById(summaryId);
                if (oldSummary != null) {
//                    OCIP-2011 开启多G6：1、在已办事项-撤销回退记录打开提示不正确；2、重新发起后数据未更新
                    if (!Strings.equals(oldSummary.getSubject(), v5Summary.getSubject())) {
                        affairManager.updateFormCollSubject(summaryId, v5Summary.getSubject());
                    }
                    colManager.updateColSummary(v5Summary);
                } else {
                    /**
                     * OCIP-2132 外系统表单事务转发后，已发事项无记录
                     * 外系统转发会发送2次交换数据（todo、forward），事务未及时提交造成主键冲突
                     * 调用DBAgent.saveAllForceFlush保存summary强制flush
                     */
                    List<com.seeyon.apps.collaboration.po.ColSummary> summaryList = new ArrayList<com.seeyon.apps.collaboration.po.ColSummary>();
                    summaryList.add(v5Summary);
                    DBAgent.saveAllForceFlush(summaryList);
//                    colManager.saveColSummary(v5Summary);
                }
            }
            if (v5Affair != null) {
                //检查是更新还是新插入
                long newAffairId = v5Affair.getId();
                CtpAffair oldAffair = colManager.getAffairById(newAffairId);
                if (oldAffair != null) {//存在即更新
                    // 保证代理人不会消失
                    v5Affair.setProxyMemberId(oldAffair.getProxyMemberId());
                    Integer summaryState = oldAffair.getSummaryState();
                    // OCIP-2194 外系统发来的事务，作为最后一个节点处理后，未显示结束标识
                    // 流程结束，保证状态不会被更改
                    if (operation != null && ColOperation.STOP == operation) {
                        v5Affair.setSummaryState(CollaborationEnum.flowState.terminate.ordinal());
                    } else {
                        if (Integer.valueOf(CollaborationEnum.flowState.finish.ordinal()).equals(summaryState)) {
                            v5Affair.setSummaryState(summaryState);
                        }
                    }
                    if(Strings.isBlank(v5Affair.getSubject())) {
                    	v5Affair.setSubject(oldAffair.getSubject());
                    }
                    affairManager.updateAffair(v5Affair);
                } else {
                    affairManager.save(v5Affair);
                }
            }
        } catch (Throwable e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    @Override
    public void forward(Affair affair, ColSummary colSummary, String sendName, String sendSysCode) {
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        // v5Affair.setAddition(sendSysCode);
        Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
        paraMap.put("ocipSystemCode", sendSysCode);
        AffairUtil.setExtProperty(v5Affair, paraMap);
        this.saveOrUpdateSummaryAndAffair(v5Summary, v5Affair);
    }

    @Override
    public void sendZcdb(Affair affair, ColSummary colSummary) {
        try {
        	Map<String,Object> columns = new HashMap<String, Object>();
            User user = AppContext.getCurrentUser();
            Set<MessageReceiver> receivers = new HashSet<MessageReceiver>();
            String messageUrl = "message.link.col.pending";
            if (Integer.valueOf(StateEnum.col_sent.getKey()).equals(affair.getState())) {
                messageUrl = "message.link.col.waiSend";
            }
            String forwardMemberName = getForwardMemberName(affair.getForwardMember());
            receivers.add(new MessageReceiver(affair.getId(), affair.getMemberId(), messageUrl, affair.getId().toString(), null));
            userMessageManager.sendSystemMessage(new MessageContent("collaboration.opinion.zcdb", affair.getSubject(), user.getName(), forwardMemberName == null ? 0 : 1, forwardMemberName, -1, "", -1, 0)
                            .setImportantLevel(affair.getImportantLevel()),
                    ApplicationCategoryEnum.collaboration, user.getId(), receivers, affair.getImportantLevel());
            columns.put("subState", affair.getSubState());
            affairManager.update(affair.getId(), columns);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void fireDelete(Affair affair, ColSummary colSummary, String sendSysCode) {
        com.seeyon.apps.collaboration.po.ColSummary v5Summary = new com.seeyon.apps.collaboration.po.ColSummary();
        BeanUtils.convert(v5Summary, colSummary);
        CtpAffair v5Affair = new CtpAffair();
        BeanUtils.convert(v5Affair, affair);
        Map<String, Object> paraMap = AffairUtil.getExtProperty(v5Affair);
        paraMap.put("ocipSystemCode", sendSysCode);
        AffairUtil.setExtProperty(v5Affair, paraMap);
        this.saveOrUpdateSummaryAndAffair(v5Summary, v5Affair);
        //发送消息
        ArrayList<CtpAffair> ctpAffairs = new ArrayList<CtpAffair>();
        ctpAffairs.add(v5Affair);
        this.colMessageManager.superviseDelete(null, ctpAffairs);
    }

    private String getForwardMemberName(String forwardMemberId) {
        if (Strings.isNotBlank(forwardMemberId)) {
            try {
                return orgManager.getMemberById(Long.parseLong(forwardMemberId)).getName();
            } catch (Exception e) {
                log.error("", e);
            }
        }
        return null;
    }

	public DocApi getDocApi() {
		return docApi;
	}

	public void setDocApi(DocApi docApi) {
		this.docApi = docApi;
	}

    public WorkflowApiManager getWapi() {
        return wapi;
    }

    public void setWapi(WorkflowApiManager wapi) {
        this.wapi = wapi;
    }
}
