package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.collaboration.manager.ColManagerImpl;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.apps.collaboration.util.ColUtil;
import com.seeyon.apps.collaboration.vo.ColSummaryVO;
import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.apps.ocip.organization.bo.IOcipOrgEntity;
import com.seeyon.apps.ocip.util.AdvisorUtil;
import com.seeyon.apps.ocip.util.OcipAxis2Client;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.content.ContentViewRet;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.permission.vo.PermissionVO;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ocip.common.IConstant;
import com.seeyon.ocip.common.entry.Address;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.configuration.OcipConfiguration;
import com.seeyon.ocip.exchange.model.col.ColOperation;

public class OCIPColManagerImpl extends ColManagerImpl implements OCIPColManager, MethodInterceptor {
    private final static Log log = LogFactory.getLog(OCIPColManagerImpl.class);
    private OrgManager orgManager;
    private AffairManager affairManager;
    private PermissionManager permissionManager;
    private CollaborationExchangeManager collaborationExchangeManager;
    private IOrganizationManager organizationManager;
    private OCIPBizManager ocipBizManager;

    @Override
    public void setOrgManager(OrgManager orgManager) {
        super.setOrgManager(orgManager);
        this.orgManager = orgManager;
    }

    @Override
    public void setAffairManager(AffairManager affairManager) {
        super.setAffairManager(affairManager);
        this.affairManager = affairManager;
    }

    @Override
    public void setPermissionManager(PermissionManager permissionManager) {
        super.setPermissionManager(permissionManager);
        this.permissionManager = permissionManager;
    }

    public void setOrganizationManager(IOrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    public void setOcipBizManager(OCIPBizManager ocipBizManager) {
        this.ocipBizManager = ocipBizManager;
    }

    @Override
    public ColSummaryVO transShowSummary(ColSummaryVO summaryVO) throws BusinessException {

        ColSummaryVO colSummaryVO = super.transShowSummary(summaryVO);
        if (colSummaryVO != null && colSummaryVO.getSummary() != null) {
            ColSummary summary = colSummaryVO.getSummary();
            //判断是否是外系统人员ID，将发起人替换为本系统人员
            try {
                ContentViewRet contentContext = (ContentViewRet) AppContext.getRequestContext("contentContext");
                V3xOrgMember isLocalMember = orgManager.getMemberById(summary.getStartMemberId());
                if (OrgUtil.isPlatformEntity(isLocalMember)) {
                    String localMemberId = organizationManager.getLocalObjectId(IConstant.AddressType.member.name(), String.valueOf(summary.getStartMemberId()));
                    contentContext.setContentSenderId(Long.parseLong(localMemberId));
                }
            } catch (Exception e) {
                log.error(e);
            }
        }
        return colSummaryVO;
    }

    @Override
    public Map<String, String> getColDefaultNode(Long orgAccountId) throws BusinessException {
        //默认节点权限
        PermissionVO permission = permissionManager.getDefaultPermissionByConfigCategory(EnumNameEnum.col_flow_perm_policy.name(), orgAccountId);
        //ocip  外系统处理时，permission为空，改为获取系统默认节点
        if (null == permission && AppContext.hasPlugin("ocip")) {
            permission = this.permissionManager.getDefaultPermissionByConfigCategory(EnumNameEnum.col_flow_perm_policy.name(), 1l);
            Map<String, String> tempMap = new HashMap<String, String>();
            tempMap.put("defaultNodeName", permission == null ? "" : permission.getName());
            tempMap.put("defaultNodeLable", permission == null ? "" : permission.getLabel());
            return tempMap;
        } else {
            return super.getColDefaultNode(orgAccountId);
        }
    }

    /**
     * 取回
     *
     * @param ma
     * @return
     * @throws Throwable
     */
    @Override
    public String transTakeBack(Map<String, Object> ma) throws BusinessException {
        CtpAffair affair = affairManager.get(Long.valueOf(String.valueOf(ma.get("affairId"))));
        //ocip 跨系统取回
        String sysCode = null;
        Map<String, Object> affairExtMap = AffairUtil.getExtProperty(affair);
        if (affairExtMap != null) {
            sysCode = (String) affairExtMap.get("ocipSystemCode");
        }
        if (AppContext.hasPlugin("ocip") && sysCode != null && sysCode.startsWith("0|system")) {
            try {
                OcipOrgMember member = organizationManager.getMember(AppContext.getCurrentUser().getId().toString(), Global.getSysCode());
                if (null == member) {
                    throw new RuntimeException("当前人员可能未上报平台，请联系管理员");
                }
                String requestAddress = OcipConfiguration.getInstance().getExchangeSpi().getTransportService().requestAddress(sysCode);
                Map<String, String> systemProperties = organizationManager.getSystemProperties(Address.parse(sysCode).getId());
                String wssUsername = systemProperties.get(OCIPConstants.WSS_USERNAME);
                String wssPassword = systemProperties.get(OCIPConstants.WSS_PASSWORD);
                OcipAxis2Client client = new OcipAxis2Client("http://" + requestAddress, wssUsername, wssPassword);
                String[] invoke = client.invoke("ocipBusinessService", "takeBack", String[].class, "http://impl.webservice.ocip.apps.seeyon.com", new Object[]{ma.get("affairId").toString(), ma.get("isSaveOpinion").toString(), member.getId()});
                //处理返回结果
                String msg = null;
                if (!Strings.isEmpty(invoke[0]) && !"null".equals(invoke[0])) {
                    msg = invoke[0];
                }
                return msg;
            } catch (final Exception e) {
                return "取回跨系统事务失败:" + e.getMessage();
            }
        } else {
            return super.transTakeBack(ma);
        }
    }

    /**
     * 检测是否有转发权限
     *
     * @param affairIds
     * @return
     * @throws Throwable
     */
    public List<String> checkForwardPermission(String affairIds) throws BusinessException {
        List<String> noPermissionCols = new ArrayList<String>();
        String[] ids = affairIds.split("[,]");
        for (String id : ids) {
            long affairId = Long.parseLong(id);

            CtpAffair affair = affairManager.get(affairId);
            ColSummary summary = getColSummaryById(affair.getObjectId());
            //OCIP 来自外系统的公文/事务不能在列表上转发
            if (AppContext.hasPlugin("ocip")) {
                String ocipSystemCode = null;
                Map<String, Object> affairExtMap = AffairUtil.getExtProperty(affair);
                if (affairExtMap != null) {
                    ocipSystemCode = (String) affairExtMap.get("ocipSystemCode");
                }
                if (ocipSystemCode != null && ocipSystemCode.startsWith("0|system")) {
                    noPermissionCols.add("《" + ColUtil.showSubjectOfSummary(summary, false, -1, "") + "》");
                }
            } else {
                List<String> res = super.checkForwardPermission(id);
                if (!res.isEmpty()) {
                    noPermissionCols.addAll(res);
                }
            }
        }
        return noPermissionCols;
    }

    /**
     * 跨系统协同转办
     *
     * @param params
     */
    @Override
    public String transColTransfer(Map<String, String> params) throws BusinessException {
        String message = super.transColTransfer(params);
        Long affairId = Long.valueOf(params.get("affairId"));
        //多G6 移交 chenyq 20180823
        if (AppContext.hasPlugin("ocip")) {
            try {
                CtpAffair ctpAffair = affairManager.get(affairId);
                List<CtpAffair> affairList = ocipBizManager.getAffairList4Exchange(ctpAffair);
                if (!affairList.isEmpty()) {
                    AffairData affairData = new AffairData();
                    affairData.setAffairList(affairList);
                    ColSummary summary = getColSummaryById(ctpAffair.getObjectId());
                    collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.DONE, summary);
                }
            } catch (Exception e) {
                log.error("多G6移交出错", e);
            }
        }
        return message;
    }

    /**
     * 判断是否是外系统发过来的事务
     *
     * @param affiarId
     * @return 0 发送人判断 1处理人判断 2addition判断
     * @throws BusinessException
     */
    @Override
    public Map checkIsExchangeAffair(String affiarId, String checkType) {
        Map result = new HashMap();
        try {
            if (Strings.isNotBlank(affiarId)) {//模板检查
                Long aId = Long.parseLong(affiarId);
                CtpAffair ctpAffair = affairManager.get(aId);
                long senderId = 0L;
                if ("0".equals(checkType)) {
                    senderId = ctpAffair.getSenderId();
                } else if ("1".equals(checkType)) {
                    senderId = ctpAffair.getMemberId();
                } else if ("2".equals(checkType)) {
                    String addition = null;
                    Map<String, Object> affairExtMap = AffairUtil.getExtProperty(ctpAffair);
                    if (affairExtMap != null) {
                        addition = (String) affairExtMap.get("ocipSystemCode");
                    }
                    if (!Strings.isEmpty(addition) && addition.startsWith("0|system")) {
                        result.put("flag", "yes");
                        return result;
                    }
                }
                User currentUser = AppContext.getCurrentUser();
                V3xOrgMember orgMember = orgManager.getMemberById(senderId);
                if (OrgUtil.isPlatformEntity(orgMember) || currentUser instanceof IOcipOrgEntity) {
                    result.put("flag", "yes");
                    return result;
                }

            }
            result.put("flag", "no");
        } catch (Exception e) {
            result.put("flag", "no");
            log.error(e);
        }
        return result;
    }

    /**
     * 解决跨系统暂存待办报异常
     * org.hibernate.NonUniqueObjectException: a different object with the same identifier value was already associated with the session: [com.seeyon.apps.collaboration.po.ColSummary#-1748668800569422258]
     */
    @Override
    public void updateColSummary(ColSummary colSummary) {
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        for (StackTraceElement e : stackTrace) {
            if (e.getClassName().equals("com.seeyon.apps.collaboration.manager.ColManagerImpl") && e.getMethodName().equals("transDoZcdbPublic")) {
                User currentUser = AppContext.getCurrentUser();
                if (currentUser instanceof IOcipOrgEntity) {//ocip单点登录用户
                    DBAgent.merge(colSummary);
                    return;
                }
            }
        }
        super.updateColSummary(colSummary);
    }

    @Override
    public Object invoke(MethodInvocation methodInvocation) throws Throwable {
        return AdvisorUtil.invoke(this, methodInvocation, null);
    }
}
