package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.JoinPoint;

import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.apps.doc.api.DocApi;
import com.seeyon.apps.doc.bo.DocResourceBO;
import com.seeyon.apps.ocip.organization.bo.IOcipOrgEntity;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ocip.exchange.model.col.ColOperation;

/**
 * OCIP收藏相关切面
 */
public class OCIPFavoriteManager extends OCIPBizManagerImpl {
    private final static Log log = LogFactory.getLog(OCIPFavoriteManager.class);
    private AffairManager affairManager;
    private ColManager colManager;
    private CollaborationExchangeManager collaborationExchangeManager;
    private DocApi docApi;
    

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }


    /**
     * 跨系统收藏
     *
     * @param joinPoint
     */
    public void saveDocFavorite(JoinPoint joinPoint) {
        Integer appEnumKey = (Integer) joinPoint.getArgs()[0];
        Integer docSourceType = (Integer) joinPoint.getArgs()[7];
        if (docSourceType == 3 && AppContext.hasPlugin("ocip") && appEnumKey == ApplicationCategoryEnum.collaboration.getKey()) {
            Long sourceId = (Long) joinPoint.getArgs()[1];
            Boolean hasAttachment = (Boolean) joinPoint.getArgs()[2];
            try {
                User currentUser = AppContext.getCurrentUser();
                if (currentUser instanceof IOcipOrgEntity) {//ocip单点登录用户
                    CtpAffair ocipAffair = affairManager.get(sourceId);
                    if (ocipAffair != null) {
                        Long objectId = ocipAffair.getObjectId();
                        ColSummary summary = colManager.getColSummaryById(objectId);
                        List<CtpAffair> affairList = getAffairList4Exchange(ocipAffair);//跨系统对意见回复进行提醒
                        if (affairList != null && affairList.size() > 0) {
                            AffairData affairData = new AffairData();
                            affairData.setAffairList(affairList);
                            affairData.setState(0);//收藏新增操作
                            affairData.setModuleId(objectId);
                            affairData.setIsSendMessage(false);
                            affairData.setSubject(ocipAffair.getSubject());
                            //对收藏信息数据进行封装
                            affairData.setImportantLevel(appEnumKey);//借用封装appEnumKey
                            affairData.setFormId(sourceId);//借用封装sourceId
                            affairData.setIsHasAttachment(hasAttachment);//借用封装has
                            affairData.setResentTime(docSourceType);//借用封装favoriteType
                            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.FAVORITE, summary);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("跨系统收藏失败", e);
            }
        }
    }

    /**
     * 跨系统取消收藏
     *
     * @param joinPoint
     */
    public void favoriteCancel(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            Long docId = (Long) joinPoint.getArgs()[0];
            Long sourceId = (Long) joinPoint.getArgs()[1];
            try {
                User currentUser = AppContext.getCurrentUser();
                if (currentUser instanceof IOcipOrgEntity) {//ocip单点登录用户
                    CtpAffair ocipAffair = affairManager.get(sourceId);
                    if (ocipAffair != null) {
                        Long objectId = ocipAffair.getObjectId();
                        ColSummary summary = colManager.getColSummaryById(objectId);
                        List<CtpAffair> affairlist = getAffairList4Exchange(ocipAffair);
                        if (affairlist != null && affairlist.size() > 0) {
                            AffairData affairData = new AffairData();
                            affairData.setAffairList(affairlist);
                            affairData.setState(1);//取消操作
                            affairData.setModuleId(ocipAffair.getObjectId());
                            affairData.setIsSendMessage(false);
                            affairData.setSubject(ocipAffair.getSubject());
                            affairData.setFormId(sourceId);//借用封装sourceId
                            affairData.setCaseId(docId);//借用封装docId
                            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.FAVORITE, summary);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("跨系统取消收藏失败", e);
            }
        }
    }

    /**
     * 跨系统更新收藏
     *
     * @param joinPoint
     * @throws BusinessException 
     */
    public void updateDocFavorite(JoinPoint joinPoint) throws BusinessException {
        Long docId = (Long) joinPoint.getArgs()[0];
        DocResourceBO doc = docApi.getDocResource(docId);
        if (AppContext.hasPlugin("ocip") && doc.getFrType() == ApplicationCategoryEnum.collaboration.getKey()) {
            //用于存储提醒外系统人员意见回复事务
            List<CtpAffair> fAffairlist = new UniqueList<CtpAffair>();
            try {
                User currentUser = AppContext.getCurrentUser();
                if (currentUser instanceof IOcipOrgEntity) {//ocip单点登录用户
                    CtpAffair ocipAffair = affairManager.get(doc.getSourceId());
                    if (ocipAffair != null) {
                        Long objectId = ocipAffair.getObjectId();
                        ColSummary summary = colManager.getColSummaryById(objectId);
                        if (fAffairlist != null && fAffairlist.size() > 0) {
                            AffairData affairData = new AffairData();
                            affairData.setAffairList(fAffairlist);
                            affairData.setState(0);//收藏新增/修改操作
                            affairData.setModuleId(ocipAffair.getObjectId());
                            affairData.setIsSendMessage(false);
                            affairData.setSubject(ocipAffair.getSubject());
                            collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.FAVORITE, summary);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("跨系统更新收藏失败", e);
            }
        }
    }

	public DocApi getDocApi() {
		return docApi;
	}

	public void setDocApi(DocApi docApi) {
		this.docApi = docApi;
	}
}
