package com.seeyon.apps.ocip.exchange.collaboration.manager;

import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.JoinPoint;

import com.seeyon.apps.collaboration.manager.ColManager;
import com.seeyon.apps.collaboration.po.ColSummary;
import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ctp.workflow.event.EventDataContext;
import com.seeyon.ocip.exchange.model.col.ColOperation;

import net.joinwork.bpm.engine.wapi.WorkItem;

public class OcipWorkFlowManagerImpl extends OCIPBizManagerImpl implements OcipWorkFlowManager {
    private static Log log = LogFactory.getLog(OcipWorkFlowManagerImpl.class);
    private ColManager colManager;
    private AffairManager affairManager;
    private CollaborationExchangeManager collaborationExchangeManager;

    public void setColManager(ColManager colManager) {
        this.colManager = colManager;
    }

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setCollaborationExchangeManager(CollaborationExchangeManager collaborationExchangeManager) {
        this.collaborationExchangeManager = collaborationExchangeManager;
    }

    @Override
    public void stop(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            EventDataContext context = (EventDataContext) joinPoint.getArgs()[0];
            WorkItem workitem = context.getWorkItem();
            try {
                CtpAffair affair = affairManager.getAffairBySubObjectId(Long.valueOf(workitem.getId()));
                List<CtpAffair> affairsByObjectIdAndState = affairManager.getAffairs(affair.getObjectId(), StateEnum.col_done);

                if (affairsByObjectIdAndState != null && !affairsByObjectIdAndState.isEmpty()) {
                    List<CtpAffair> list = new UniqueList<CtpAffair>();
                    for (CtpAffair affairData : affairsByObjectIdAndState) {
                        if (affairData != null) {
                            affairData.setState(StateEnum.col_done.key());
                            affairData.setSubState(SubStateEnum.col_done_stepStop.key());
                            affairData.setCompleteTime(new Date());//设置处理时间
                            list.add(affairData);
                        }
                    }
                    list = getAffairList4Exchange(list);
                    //跨系统对超期提醒人员进行分发
                    if (!list.isEmpty()) {
                        AffairData affairData = new AffairData();
                        affairData.setAffairList(list);
                        affairData.setIsSendMessage(false);
                        affairData.setSubject(affair.getSubject());
                        affairData.setModuleId(affair.getObjectId());
                        ColSummary doneSummary = colManager.getColSummaryById(affair.getObjectId());
                        collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.STOP, doneSummary);
                    }
                }
            } catch (Exception e) {
                log.error("流程终止同步外系统异常", e);
            }
        }
    }

    @Override
    public void doneToReady(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            EventDataContext context = (EventDataContext) joinPoint.getArgs()[0];
            List<CtpAffair> list = new UniqueList<CtpAffair>();
            List<WorkItem> workItems = context.getWorkitemLists();
            Long summaryId = -1L;
            try {
                for (WorkItem wi : workItems) {
                    CtpAffair affair = affairManager.getAffairBySubObjectId(wi.getId());
                    if (affair != null) {
                        list.add(affair);
                    }
                }
                list = getAffairList4Exchange(list);
                if (list.size() > 0) {
                    ColSummary colSummary = colManager.getColSummaryById(summaryId);
                    AffairData affairData = new AffairData();
                    affairData.setAffairList(list);
                    affairData.setSubject(colSummary.getSubject());
                    affairData.setIsSendMessage(false);
                    collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.TODO, colSummary);
                }
            } catch (Exception e) {
                log.error("流程已办转待办同步外系统异常", e);
            }
        }
    }

    @Override
    public void canceled(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            List<WorkItem> workitems = (List<WorkItem>) joinPoint.getArgs()[0];
            if (workitems != null && !workitems.isEmpty()) {
                for (WorkItem item : workitems) {
                    sendExchangeSuperviseCancel(item.getId());
                }
            }
        }
    }

    /**
     * 判断是否要推送督办替换节点至外单位修改ctp_affair表的状态
     *
     * @param subObjectId
     */
    private void sendExchangeSuperviseCancel(long subObjectId) {
        try {
            //用于督办替换节点至外单位的事务
            CtpAffair affair = affairManager.getAffairBySubObjectId(subObjectId);
            if (affair != null) {
                //对于提醒发起人，标识是提醒发起人
                affair.setState(StateEnum.col_cancel.key());
                affair.setSubState(SubStateEnum.col_normal.key());
                affair.setDelete(true);
                List<CtpAffair> list = getAffairList4Exchange(affair);
                //跨系统对超期提醒人员进行分发
                if (list != null && list.size() > 0) {
                    AffairData affairData = new AffairData();
                    affairData.setAffairList(list);
                    affairData.setIsSendMessage(false);
                    affairData.setSubject("督办替换节点修改外系统事务状态");
                    ColSummary colSummary = colManager.getColSummaryById(affair.getObjectId());
                    collaborationExchangeManager.handCollaborationAffair(affairData, ColOperation.DONE, colSummary);
                }
            }
        } catch (Exception e) {
            log.error("推送外系统督办失败", e);
        }
    }
}
